//---------------------------------------------------------------------------

#pragma hdrstop

#include "RS.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
using namespace std;

__fastcall RS::RS()
{
        NPAR=4;
        MAXDEG=2*NPAR;
        DEBUG = FALSE;
        pBytes=new int[MAXDEG];
        synBytes=new int[MAXDEG];
        genPoly=new int[2*MAXDEG];
        Lambda=new int[MAXDEG];
        Omega=new int[MAXDEG];
        initialize_ecc(NPAR);
}

void __fastcall RS::initialize_ecc(int NumberOfParityBytes)
{

        if(NumberOfParityBytes!=NPAR)
        {
                delete [] pBytes;
                delete [] synBytes;
                delete [] genPoly;
                delete [] Lambda;
                delete [] Omega;
                NPAR=NumberOfParityBytes;
                MAXDEG=2*NPAR;
                pBytes=new int[MAXDEG];
                synBytes=new int[MAXDEG];
                genPoly=new int[2*MAXDEG];
                Lambda=new int[MAXDEG];
                Omega=new int[MAXDEG];
        }

        /* Initialize the galois field arithmetic tables */
        init_galois_tables();

        /* Compute the encoder generator polynomial */
        compute_genpoly(NPAR, genPoly);
}

void __fastcall RS::init_galois_tables()
{
        /* initialize the table of powers of alpha */
        init_exp_table();
}

void __fastcall RS::init_exp_table()
{
        int i, z;
        int pinit,p1,p2,p3,p4,p5,p6,p7,p8;

        pinit = p2 = p3 = p4 = p5 = p6 = p7 = p8 = 0;
        p1 = 1;

        gexp[0] = 1;
        gexp[255] = gexp[0];
        glog[0] = 0;			/* shouldn't log[0] be an error? */

        for (i = 1; i < 256; i++)
        {
                pinit = p8;
                p8 = p7;
                p7 = p6;
                p6 = p5;
                p5 = p4 ^ pinit;
                p4 = p3 ^ pinit;
                p3 = p2 ^ pinit;
                p2 = p1;
                p1 = pinit;
                gexp[i] = p1 + p2*2 + p3*4 + p4*8 + p5*16 + p6*32 + p7*64 + p8*128;
                gexp[i+255] = gexp[i];
        }
	
        for (i = 1; i < 256; i++)
        {
                for (z = 0; z < 256; z++)
                {
                        if (gexp[z] == i)
                        {
	                        glog[i] = z;
	                        break;
                        }
                }
        }
}

/* multiplication using logarithms */
int __fastcall RS::gmult(int a, int b)
{
        int i,j;
        if (a==0 || b == 0) return (0);
        i = glog[a];
        j = glog[b];
        return (gexp[i+j]);
}

int __fastcall RS::ginv(int elt)
{
        return (gexp[255-glog[elt]]);
}

/* Create a generator polynomial for an n byte RS code.
 * The coefficients are returned in the genPoly arg.
 * Make sure that the genPoly array which is passed in is
 * at least n+1 bytes long.
 */
void __fastcall RS::compute_genpoly(int nbytes, int genpoly[])
{
        int i, tp[256], tp1[256];
        /* multiply (x + a^n) for n = 1 to nbytes */
        zero_poly(tp1);
        tp1[0] = 1;
        for (i = 1; i <= nbytes; i++)
        {
                zero_poly(tp);
                tp[0] = gexp[i];		/* set up x+a^n */
                tp[1] = 1;

                mult_polys(genpoly, tp, tp1);
                copy_poly(tp1, genpoly);
        }
}

/* polynomial multiplication */
void __fastcall RS::mult_polys(int dst[], int p1[], int p2[])
{
        int i, j;
        int *tmp1=new int[MAXDEG*2];
        for (i=0; i < (MAXDEG*2); i++) dst[i] = 0;
        for (i = 0; i < MAXDEG; i++)
        {
                for(j=MAXDEG; j<(MAXDEG*2); j++) tmp1[j]=0;
                // scale tmp1 by p1[i]
                for(j=0; j<MAXDEG; j++) tmp1[j]=gmult(p2[j], p1[i]);
                // and mult (shift) tmp1 right by i
                for (j = (MAXDEG*2)-1; j >= i; j--) tmp1[j] = tmp1[j-i];
                for (j = 0; j < i; j++) tmp1[j] = 0;
                // add into partial product
                for(j=0; j < (MAXDEG*2); j++) dst[j] ^= tmp1[j];
        }
        delete [] tmp1;
}

void __fastcall RS::zero_poly(int poly[])
{
        int i;
        for (i = 0; i < MAXDEG; i++) poly[i] = 0;
}

void __fastcall RS::copy_poly(int dst[], int src[])
{
        int i;
        for (i = 0; i < MAXDEG; i++) dst[i] = src[i];
}


/* Simulate a LFSR with generator polynomial for n byte RS code. 
 * Pass in a pointer to the data array, and amount of data. 
 *
 * The parity bytes are deposited into pBytes[], and the whole message
 * and parity are copied to dest to make a codeword.
 * 
 */
void __fastcall RS::encode_data(unsigned char msg[], int nbytes, unsigned char dst[])
{
        int i,dbyte, j;
        int *LFSR=new int[NPAR+1];
        for(i=0; i < NPAR+1; i++) LFSR[i]=0;
        for (i = 0; i < nbytes; i++)
        {
                dbyte = msg[i] ^ LFSR[NPAR-1];
                for (j = NPAR-1; j > 0; j--)
                {
                        LFSR[j] = LFSR[j-1] ^ gmult(genPoly[j], dbyte);
                }
                LFSR[0] = gmult(genPoly[0], dbyte);
        }

        for (i = 0; i < NPAR; i++) pBytes[i] = LFSR[i];
        build_codeword(msg, nbytes, dst);
        delete [] LFSR;
}

/* Append the parity bytes onto the end of the message */
void __fastcall RS::build_codeword(unsigned char msg[], int nbytes, unsigned char dst[])
{
        int i;
        for (i = 0; i < nbytes; i++) dst[i] = msg[i];
        for (i = 0; i < NPAR; i++) dst[i+nbytes] = pBytes[NPAR-1-i];
}

/**********************************************************
 * Reed Solomon Decoder 
 *
 * Computes the syndrome of a codeword. Puts the results
 * into the synBytes[] array.
 */
void __fastcall RS::decode_data(unsigned char data[], int nbytes)
{
        int i, j, sum;
        for (j = 0; j < NPAR;  j++)
        {
                sum = 0;
                for (i = 0; i < nbytes; i++)
                {
                        sum = data[i] ^ gmult(gexp[j+1], sum);
                }
                synBytes[j]  = sum;
        }
        for(j=NPAR;j<MAXDEG;j++)synBytes[j]=0;
}

void __fastcall RS::decode_data(int data[], int nbytes)
{
        int i, j, sum, tmp;
        for (j = 0; j < NPAR;  j++)
        {
                sum = 0;
                for (i = 0; i < nbytes; i++)
                {
                        tmp = data[i];
                        if(tmp<0)tmp=0;
                        sum = (unsigned char)(tmp ^ gmult(gexp[j+1], sum));
                        //sum = (unsigned char)tmp ^ gmult(gexp[j+1], sum);
                }
                synBytes[j]  = sum;
        }
        for(j=NPAR;j<MAXDEG;j++)synBytes[j]=0;
}

int __fastcall RS::check_syndrome()
{
        int i, nz = 0;
        for (i =0 ; i < NPAR; i++)
        {
                if (synBytes[i] != 0) nz = 1;
        }
        return nz;
}

/* Combined Erasure And Error Magnitude Computation
 *
 * Pass in the codeword, its size in bytes, as well as
 * an array of any known erasure locations, along the number
 * of these erasures.
 *
 * Evaluate Omega(actually Psi)/Lambda' at the roots
 * alpha^(-i) for error locs i.
 *
 * Returns 1 if everything ok, or 0 if an out-of-bounds error is found
 *
 */
int __fastcall RS::correct_errors_erasures(unsigned char codeword[], int csize, int nerasures, int erasures[])
{
        int r, i, j, err;
        /* If you want to take advantage of erasure correction, be sure to
           set NErasures and ErasureLocs[] with the locations of erasures.
        */
        NErasures = nerasures;
        for (i = 0; i < NErasures; i++) ErasureLocs[i] = erasures[i];

        Modified_Berlekamp_Massey();
        Find_Roots();


        if ((NErrors <= NPAR) && NErrors > 0)
        {
                UncorrectableCodeword=false;
                /* first check for illegal error locs */
                for (r = 0; r < NErrors; r++)
                {
                        if (ErrorLocs[r] >= csize)
                        {
	                        //if (DEBUG) fprintf(stderr, "Error loc i=%d outside of codeword length %d\n", i, csize);
	                        return(0);
                        }
                }

                for (r = 0; r < NErrors; r++)
                {
                        int num, denom;
                        i = ErrorLocs[r];
                        /* evaluate Omega at alpha^(-i) */

                        num = 0;
                        for (j = 0; j < MAXDEG; j++) num ^= gmult(Omega[j], gexp[((255-i)*j)%255]);

                        /* evaluate Lambda' (derivative) at alpha^(-i) ; all odd powers disappear */
                        denom = 0;
                        for (j = 1; j < MAXDEG; j += 2)
                        {
	                        denom ^= gmult(Lambda[j], gexp[((255-i)*(j-1)) % 255]);
                        }

                        err = gmult(num, ginv(denom));
                        //if (DEBUG) fprintf(stderr, "Error magnitude %#x at loc %d\n", err, csize-i);

                        codeword[csize-i-1] ^= err;
                }
                return(1);
        }
         else
         {
                //if (DEBUG && NErrors) fprintf(stderr, "Uncorrectable codeword\n");
                UncorrectableCodeword=true;
                return(0);
         }
}

int __fastcall RS::correct_errors_erasures(int codeword[], int csize)
{
        int r, i, j, err,yy;
        unsigned char uc;
        /* If you want to take advantage of erasure correction, be sure to
           set NErasures and ErasureLocs[] with the locations of erasures.
        */

        NErasures = 0;
        for(i=0;i<csize;i++)
        {
                if(codeword[i]<0)
                {
                        codeword[i]=0;
                        ErasureLocs[NErasures] = i;
                        NErasures++;
                }
        }

        //for (i = 0; i < NErasures; i++) ErasureLocs[i] = erasures[i];

        Modified_Berlekamp_Massey();
        Find_Roots();


        if ((NErrors <= NPAR) && NErrors > 0)
        {
                UncorrectableCodeword=false;
                /* first check for illegal error locs */
                for (r = 0; r < NErrors; r++)
                {
                        if (ErrorLocs[r] >= csize)
                        {
	                        //if (DEBUG) fprintf(stderr, "Error loc i=%d outside of codeword length %d\n", i, csize);
	                        return(0);
                        }
                }

                for (r = 0; r < NErrors; r++)
                {
                        int num, denom;
                        i = ErrorLocs[r];
                        /* evaluate Omega at alpha^(-i) */

                        num = 0;
                        for (j = 0; j < MAXDEG; j++) num ^= gmult(Omega[j], gexp[((255-i)*j)%255]);

                        /* evaluate Lambda' (derivative) at alpha^(-i) ; all odd powers disappear */
                        denom = 0;
                        for (j = 1; j < MAXDEG; j += 2)
                        {
	                        denom ^= gmult(Lambda[j], gexp[((255-i)*(j-1)) % 255]);
                        }

                        err = gmult(num, ginv(denom));
                        //if (DEBUG) fprintf(stderr, "Error magnitude %#x at loc %d\n", err, csize-i);

                        codeword[csize-i-1] =  (codeword[csize-i-1] ^ err)&255;
                }
                return(1);
        }
         else
         {
                //if (DEBUG && NErrors) fprintf(stderr, "Uncorrectable codeword\n");
                UncorrectableCodeword=true;
                return(0);
         }
}

void __fastcall RS::Modified_Berlekamp_Massey()
{
        int n, L, L2, k, d, i;
        int *psi=new int[MAXDEG];
        int *psi2=new int[MAXDEG];
        int *D=new int[MAXDEG];
        int *gamma=new int[MAXDEG];
        /* initialize Gamma, the erasure locator polynomial */
        init_gamma(gamma);
        /* initialize to z */
        copy_poly(D, gamma);
        mul_z_poly(D);
        copy_poly(psi, gamma);
        k = -1; L = NErasures;
        for (n = NErasures; n < NPAR; n++)
        {
                d = compute_discrepancy(psi, synBytes, L, n);
                if (d != 0)
                {
                        /* psi2 = psi - d*D */
                        for (i = 0; i < MAXDEG; i++) psi2[i] = psi[i] ^ gmult(d, D[i]);
                        if (L < (n-k))
                        {
	                        L2 = n-k;
	                        k = n-L;
	                        /* D = scale_poly(ginv(d), psi); */
	                        for (i = 0; i < MAXDEG; i++) D[i] = gmult(psi[i], ginv(d));
	                        L = L2;
                        }
                        /* psi = psi2 */
                        for (i = 0; i < MAXDEG; i++) psi[i] = psi2[i];
                }
                mul_z_poly(D);
        }
        for(i = 0; i < MAXDEG; i++) Lambda[i] = psi[i];
        compute_modified_omega();
        delete [] D;
        delete [] gamma;
        delete [] psi;
        delete [] psi2;
}

/* given Psi (called Lambda in Modified_Berlekamp_Massey) and synBytes,
   compute the combined erasure/error evaluator polynomial as
   Psi*S mod z^4
  */
void __fastcall RS::compute_modified_omega()
{
        int i;
        int *product=new int[MAXDEG*2];
        mult_polys(product, Lambda, synBytes);
        zero_poly(Omega);
        for(i = 0; i < NPAR; i++) Omega[i] = product[i];
        delete [] product;
}

/* gamma = product (1-z*a^Ij) for erasure locs Ij */
void __fastcall RS::init_gamma(int gamma[])
{
        int e;
        int *tmp=new int[MAXDEG];
        zero_poly(gamma);
        zero_poly(tmp);
        gamma[0] = 1;
        for (e = 0; e < NErasures; e++)
        {
                copy_poly(tmp, gamma);
                scale_poly(gexp[ErasureLocs[e]], tmp);
                mul_z_poly(tmp);
                add_polys(gamma, tmp);
        }
        delete [] tmp;
}

int __fastcall RS::compute_discrepancy(int lambda[], int S[], int L, int n)
{
        int i, sum=0;
        for (i = 0; i <= L; i++)
        sum ^= gmult(lambda[i], S[n-i]);
        return (sum);
}

void __fastcall RS::add_polys(int dst[], int src[])
{
        int i;
        for (i = 0; i < MAXDEG; i++) dst[i] ^= src[i];
}

void __fastcall RS::scale_poly(int k, int poly[])
{
        int i;
        for (i = 0; i < MAXDEG; i++) poly[i] = gmult(k, poly[i]);
}

/* multiply by z, i.e., shift right by 1 */
void __fastcall RS::mul_z_poly(int src[])
{
        int i;
        for (i = MAXDEG-1; i > 0; i--) src[i] = src[i-1];
        src[0] = 0;
}

/* Finds all the roots of an error-locator polynomial with coefficients
 * Lambda[j] by evaluating Lambda at successive values of alpha.
 * 
 * This can be tested with the decoder's equations case.
 */
void __fastcall RS::Find_Roots()
{
        int sum, r, k;
        NErrors = 0;
        for (r = 1; r < 256; r++)
        {
                sum = 0;
                /* evaluate lambda at r */
                for (k = 0; k < NPAR+1; k++)
                {
                        sum ^= gmult(gexp[(k*r)%255], Lambda[k]);
                }
                if (sum == 0)
                {
	                ErrorLocs[NErrors] = (255-r); NErrors++;
	                //if (DEBUG) fprintf(stderr, "Root found at r = %d, (255-r) = %d\n", r, (255-r));
                }
        }
}

__fastcall RS::~RS()
{
        delete [] pBytes;
        delete [] synBytes;
        delete [] genPoly;
        delete [] Lambda;
        delete [] Omega;
}

bool __fastcall RSWrapper::GetNextEnCodedBlock()
{
        bool GotEnCodedDataBlock;
        while((EncodeBuffer.InBufferPtr<EncodeBuffer.InBuffer.size())&&(EncodeBuffer.buffptr<EncodeBuffer.buffsize))
        {
                EncodeBuffer.InBuffer[EncodeBuffer.InBufferPtr]=EncodeBuffer.buff[EncodeBuffer.buffptr];
                EncodeBuffer.InBufferPtr++;
                EncodeBuffer.buffptr++;
        }

        if(EncodeBuffer.InBufferPtr>=EncodeBuffer.InBuffer.size())
        {
                // 5 means leave enough space for header code

                //encode block
                Rs.encode_data(&EncodeBuffer.InBuffer[0],EncodeBuffer.InBuffer.size(),&EncodeBuffer.OutBuffer[5]);
                EncodeBuffer.InBufferPtr=0;

                //interleave block
                intleave.ResetInterleaverPtr();
                for(int i=5;i<EnCodedBlockSize;i++)EncodeBuffer.OutBuffer.at(i)=intleave.InterleaveUpdate(EncodeBuffer.OutBuffer.at(i));

                //add block sequence code (uses 1.7%)
                for(int i=0;i<5;i++)EncodeBuffer.OutBuffer[i]=4*EnBlockSequenceNumber+i;
                EnBlockSequenceNumber++;EnBlockSequenceNumber%=63;

                GotEnCodedDataBlock=true;
        }
         else GotEnCodedDataBlock=false;
        return GotEnCodedDataBlock;
}

bool __fastcall RSWrapper::LoadDataToEnCode(unsigned char *buf, int Length)
{
        EncodeBuffer.buffptr=0;
        EncodeBuffer.buff=buf;
        EncodeBuffer.buffsize=Length;

        EnCodedHolingDataSize=0;

        while(GetNextEnCodedBlock())
        {
                if((int)EnCodedHolingData.size()<(EnCodedHolingDataSize+EnCodedBlockSize))EnCodedHolingData.resize(EnCodedHolingDataSize+10*EnCodedBlockSize);
                int x=rand()%10000;
                //if(x>200)
                if(!insertanerror)
                {
                        for(int i=0;i<EnCodedBlockSize;i++)
                        {
                                EnCodedHolingData[EnCodedHolingDataSize]=EncodeBuffer.OutBuffer[i];
                                EnCodedHolingDataSize++;
                        }

                }
                 else insertanerror=false;
        }

        if(EnCodedHolingDataSize==0)return false;
         else return true;
}


bool __fastcall RSWrapper::LoadDataToDeCode(unsigned char *buf, int Length)
{

        DeCodedHolingDataSize=0;

        //append data to be encoded to our processing buffer
        if(((DecodeBuffer.InBufferPtr)+(unsigned int)Length)<(DecodeBuffer.InBuffer.size()))DecodeBuffer.InBuffer.resize(DecodeBuffer.InBufferPtr+Length+5000);
        for(int i=0;i<Length;i++)DecodeBuffer.InBuffer[DecodeBuffer.InBufferPtr+i]=buf[i];
        DecodeBuffer.InBufferPtr+=Length;

        //find next block of good data to decode
        int TemporyInIntBuf[256];
        unsigned char TemporyInUCBuf[256];

        int Erasures[256];
        int ErasureCount;

        int Ptr=0;
        int Header1,Header2;
        bool MissedBlock=false;
        while((Ptr+5+EnCodedBlockSize)<(int)DecodeBuffer.InBufferPtr)
        {
                Header1=DecodeHeader(&DecodeBuffer.InBuffer[Ptr]);
                Header2=DecodeHeader(&DecodeBuffer.InBuffer[Ptr+EnCodedBlockSize]);
                if((Header1!=-1)&&(Header2!=-1)&&(((Header1+1)%63)==Header2))
                {
                        //got block here

                        //place into tmp buffer so as not to modify DecodeBuffer.InBuffer
                        for(int i=0;i<(EnCodedBlockSize-5);i++)TemporyInIntBuf[i]=(int)DecodeBuffer.InBuffer[Ptr+5+i];

                        //Deal with missing blocks by padding the deinterleaver
                        LastRecivedBlock++;LastRecivedBlock%=63;
                        if(LastRecivedBlock!=Header1)
                        {

                                blocks[blockptr]=LastRecivedBlock;
                                blockptr++;blockptr%=1000;

                                MissedBlock=true;
                                for(int i=0;i<(EnCodedBlockSize-5);i++)TemporyInIntBuf[i]=-1;// erasures theses bytes are missing;
                        }

                        //deinterleave block
                        ErasureCount=0;
                        deintleave.ResetDeInterleaverPtr();
                        for(int i=0;i<(EnCodedBlockSize-5);i++)
                        {
                                TemporyInIntBuf[i]=deintleave.DeInterleaveUpdate(TemporyInIntBuf[i]);
                                if(TemporyInIntBuf[i]<0)
                                {
                                        TemporyInUCBuf[i]=0;
                                        Erasures[ErasureCount]=i;   //doesnt really seem to do much
                                        ErasureCount++;
                                }
                                 else TemporyInUCBuf[i]=TemporyInIntBuf[i];
                        }

                        //decode block

                      /*  ErasureCount=0;
                        for(int i=0;i<(EnCodedBlockSize-5);i++)
                        {
                                if(TemporyInIntBuf[i]<0)
                                {
                                        TemporyInUCBuf[i]=0;
                                        Erasures[ErasureCount]=i;
                                        ErasureCount++;
                                }
                                 else testb[i]=TemporyInBuf[i];
                        }       */

                        Rs.decode_data(TemporyInUCBuf,(EnCodedBlockSize-5));
                        //Rs.decode_data(TemporyInIntBuf,(EnCodedBlockSize-5));

                        ErrorProb.Sum-=ErrorProb.Buf[ErrorProb.BufPtr];
                        if(Rs.check_syndrome()!=0)
                        {
                                tmpcnt++;
                                Rs.correct_errors_erasures(TemporyInUCBuf,(EnCodedBlockSize-5),ErasureCount,Erasures);//0,NULL);
                                //Rs.correct_errors_erasures(TemporyInIntBuf,(EnCodedBlockSize-5));
                                ErrorProb.Buf[ErrorProb.BufPtr]=Rs.GetNErrors;
                        }
                         else ErrorProb.Buf[ErrorProb.BufPtr]=0;
                        ErrorProb.Sum+=ErrorProb.Buf[ErrorProb.BufPtr];
                        ErrorProb.BufPtr++;ErrorProb.BufPtr%=ErrorProb.Buf.size();
                        ProbabilityOfABlockHavingAnError=((double)ErrorProb.Sum)/((double)(ErrorProb.Buf.size()*EnCodedBlockSize));

                        //append decoded block to holding buffer
                        if((int)DeCodedHolingData.size()<(DeCodedHolingDataSize+DeCodedBlockSize))DeCodedHolingData.resize(DeCodedHolingDataSize+10*DeCodedBlockSize);
                        for(int i=0;i<DeCodedBlockSize;i++)
                        {
                                DeCodedHolingData.at(DeCodedHolingDataSize)=TemporyInUCBuf[i];//(unsigned char)TemporyInIntBuf[i];
                                DeCodedHolingDataSize++;
                        }
                        Ptr--;
                        if(!MissedBlock)Ptr+=EnCodedBlockSize;
                         else MissedBlock=false;

                }
                Ptr++;
        }

        //move unprocessed data to the start of our processing buffer
        for(int i=Ptr;i<(int)DecodeBuffer.InBufferPtr;i++)DecodeBuffer.InBuffer.at(i-Ptr)=DecodeBuffer.InBuffer.at(i);
        DecodeBuffer.InBufferPtr-=Ptr;

        if(DeCodedHolingDataSize==0)return false;
         else return true;

}

__fastcall RSWrapper::RSWrapper()
{
        tmpcnt=0;

        EnCodedHolingData.resize(5000);
        DeCodedHolingData.resize(5000);
        Init(223,32);
        ErrorProb.Buf.resize(1000);
}

void __fastcall RSWrapper::Init(int DataBlockSize, int NumberOfParityBytes)
{

        EncodeBuffer.InBufferPtr=0;
        EncodeBuffer.OutBufferPtr=0;
        DecodeBuffer.OutBufferPtr=0;

        DecodeBuffer.InBuffer.resize(20000);
        DecodeBuffer.InBufferPtr=0;

        Rs.NumberOfParityBytes=NumberOfParityBytes;
        DeCodedBlockSize=DataBlockSize;
        EnCodedBlockSize=DataBlockSize+NumberOfParityBytes+5;
        EncodeBuffer.InBuffer.resize(DataBlockSize);
        EncodeBuffer.OutBuffer.resize(NumberOfParityBytes+DataBlockSize+5);
        intleave.Init(DataBlockSize+NumberOfParityBytes,1);
        deintleave.Init(DataBlockSize+NumberOfParityBytes,1);
        NumberOfDetectedErrors=0;
        ErrorProb.BufPtr=0;
        ErrorProb.Sum=0;
        for(unsigned int i=0;i<ErrorProb.Buf.size();i++)ErrorProb.Buf[i]=0;
}

bool __fastcall RSWrapper::ReturnSyncPositionChanged()
{
        if(SyncPositionChanged)
        {
                SyncPositionChanged=false;
                return true;
        }
         else return false;
}

bool __fastcall RSWrapper::ReturnHaveErrorsHappened()
{
        if(DataError)
        {
                DataError=false;
                return true;
        }
         else return false;
}


int __fastcall RSWrapper::CalculateNextEnCodedDataSize(int SizeOfNextDataToEnCode)
{
        return (EnCodedBlockSize*((SizeOfNextDataToEnCode+EncodeBuffer.InBufferPtr)/DeCodedBlockSize));
}

//returns -1 if header is bad else returns the number that the header contains
int __fastcall RSWrapper::DecodeHeader(unsigned char * Header)
{
        int i,j,k;
        int r[5];
        int Result=-1;
        int count;
        for(i=0;i<5;i++)
        {
                k=((int)Header[i])-i;
                r[i]=k>>2;
                if((r[i]<<2)!=k)r[i]=-1;
        }
        for(i=0;i<3;i++)
        {
                count=1;
                for(j=i+1;j<5;j++)if(r[i]==r[j])count++;
                if(count>=3)
                {
                        Result=r[i];
                        break;
                }
        }
        return Result;
}
